from pykeyboard import InlineButton, InlineKeyboard
from pyrogram import filters
from pyrogram.client import Client
from pyrogram.types import CallbackQuery, Message

from classes.pyroclient import PyroClient


@Client.on_message(
    ~filters.scheduled & filters.private & filters.command(["language"], prefixes=["/"])  # type: ignore
)
async def command_language(app: PyroClient, message: Message):
    user = await app.find_user(message.from_user)
    keyboard = InlineKeyboard(row_width=2)
    buttons = []

    for locale, data in app.in_every_locale("metadata").items():
        buttons.append(
            InlineButton(f"{data['flag']} {data['name']}", f"language:{locale}")
        )

    keyboard.add(*buttons)

    await message.reply_text(
        app._("locale_choice", "message", locale=user.locale),
        reply_markup=keyboard,
    )


@Client.on_callback_query(filters.regex(r"language:[\s\S]*"))  # type: ignore
async def callback_language(app: PyroClient, callback: CallbackQuery):
    user = await app.find_user(callback.from_user)
    language = str(callback.data).split(":")[1]

    await user.update_locale(language)

    await callback.answer(
        app._("locale_set", "callback", locale=language).format(
            locale=app._("name", "metadata", locale=language)
        ),
        show_alert=True,
    )
